/*
this is part of brot_core.dll, a fractal generation library
Copyright (C) 2003 Rolf Neuberger

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

/*
changes:
03/Mar-05 * added proper LGPL notice, started changelog - Rolf

*/

//stuff for understanding headers
#include <vector>           //needed for MaskedImage.h
#include <deque>			//needed for Generator.h
#include "common.h"
#include "ExtremeFloat.h"	//needed for Fractal.h
#include "Section.h"		//needed for MaskedImage.h
#include "Span.h"			//needed for MaskedImage.h
#include "MaskedImage.h"	//needed for Fractal.h
#include "Fractal.h"		//needed for Generator.h
#include "Generator.h"   	//needed for Dumb.h
#include "Dumb.h"

//internal stuff
#include "x86_timer.ah"		//needed for Timing.h"
#include "Timing.h"

DumbGenerator::DumbGenerator(Fractal* fractal):
Generator(fractal)
{
	reset();
}

void
DumbGenerator::reset()
{
	width=fractal->get_width();
	height=fractal->get_height();

	current_line=0;
	current_column=0;
	reset_preview();
}

bool
DumbGenerator::refine_image(float max_time)
{
	fractal->ready();
	if (current_line==height) return(false);
	AbsTimer refine_timer;

	bool ret=preview(max_time);

	while (refine_timer.elapsed_seconds()<max_time)
	{
		fractal->sample(current_column,current_line);
		ret=true;
		++current_column;
		if (current_column==width)
		{
			current_column=0;
			++current_line;
			if (current_line==height)
			{
				break;
			}
		}
	}
	return(ret);
}

float
DumbGenerator::get_progress()
{
	if (height>0)	return(float(current_line)/height);
	else			return(0.0f);
}
