/*
this is part of brot_core.dll, a fractal generation library
Copyright (C) 2003 Rolf Neuberger

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

/*
changes:
03/Mar-05 * Bugfix: save zeroed statistics when discarding progress - Rolf
03/Mar-05 * added save member function and FILE* constructor - Rolf
03/Mar-05 * new file - Rolf
*/

#include <stdio.h>
#include <vector>
#include <deque>
#include "common.h"
#include "Section.h"
#include "Span.h"
#include "MaskedImage.h"
#include "ExtremeFloat.h"
#include "Fractal.h"
#include "Generator.h"
#include "brot_core_interface.h"
#include "HistEntry.h"

HistEntry::HistEntry(FILE* input):
generator(NULL)		//not stored
{
	char version[16];
	fgets(version,16,input);
	if (0!=strcmp(version,"v000\n"))
	{
//FIXME
	}
	else
	{
		fread(&width,sizeof(int),1,input);
		fread(&height,sizeof(int),1,input);
		fread(&aa_strength,sizeof(int),1,input);
		fread(&generator_type,sizeof(int),1,input);
		fread(&cleverness,sizeof(float),1,input);
		fread(&progress,sizeof(float),1,input);
		fread(&total_time,sizeof(float),1,input);
		fread(&finished,sizeof(bool),1,input);
		fread(&hibernated,sizeof(bool),1,input);
	}

	char signature[16];
	fgets(signature,16,input);
	if (0!=strcmp(signature,"Fractal\n"))
	{
//FIXME
		fractal=NULL;
	}
	else
	{
		fractal=new Fractal(input);
	}
}



bool
HistEntry::save(FILE* of,bool everything)
{
	fputs("v000\n",of);
	fwrite(&width,sizeof(int),1,of);
	fwrite(&height,sizeof(int),1,of);
	fwrite(&aa_strength,sizeof(int),1,of);
	fwrite(&generator_type,sizeof(int),1,of);
	if (everything)
	{
		fwrite(&cleverness,sizeof(float),1,of);
		fwrite(&progress,sizeof(float),1,of);
		fwrite(&total_time,sizeof(float),1,of);
		fwrite(&finished,sizeof(bool),1,of);
  	}
  	else
  	{
  		//the statistics are wrong if we don't write all results
  		float spam=0.0f;
  		bool dummy=false;
		fwrite(&spam,sizeof(float),1,of);
		fwrite(&spam,sizeof(float),1,of);
		fwrite(&spam,sizeof(float),1,of);
		fwrite(&dummy,sizeof(bool),1,of);
  	}
	fwrite(&hibernated,sizeof(bool),1,of);

	if (fractal)
	{
		fputs("Fractal\n",of);
		return(fractal->save(of,everything));
	}	
	else			//should not happen
	{
		fputs("NULL\n",of);
		return(false);
	}
}

