/*
this is part of brot_core.dll, a fractal generation library
Copyright (C) 2003 Rolf Neuberger

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

/*
changes:
03/Mar-06 * made weights static and abandoned std::sort for compiler compatability reasons - Rolf
03/Mar-06 * new file - Rolf
*/

#include <vector>
#include <list>
#include "common.h"
#include "Huffman.h"


class
TreeP
{
private:
	TreeP();
public:
	TreeP(Tree* p):p(p) {};

	bool operator < (const TreeP& rhs)const {return(p->weight<rhs.p->weight);}
	bool operator > (const TreeP& rhs)const {return(p->weight>rhs.p->weight);}

	Tree* p;
};


//static huffman weights. precalculated for compatibility reasons ... (see below)
static const uint weight[]={
1000000, 367879, 135335, 49787, 44255, 39829, 36208, 33191, 30638, 28449,
26553, 24893, 23429, 22127, 20962, 19914, 18966, 18104, 17317, 16595, 15931,
15319, 14751, 14224, 13734, 13276, 12848, 12446, 12069, 11714, 11379, 11063,
10764, 10481, 10212, 9957, 9714, 9483, 9262, 9052, 8851, 8658, 8474, 8297,
8128, 7965, 7809, 7659, 7515, 7375, 7241, 7112, 6987, 6867, 6750, 6638, 6529,
6424, 6322, 6223, 6127, 6034, 5944, 5857, 5772, 5689, 5609, 5531, 5456, 5382,
5310, 5240, 5172, 5106, 5041, 4978, 4917, 4857, 4798, 4741, 4685, 4631, 4578,
4526, 4475, 4425, 4376, 4329, 4282, 4237, 4192, 4148, 4106, 4064, 4023, 3982,
3943, 3904, 3866, 3829, 3793, 3757, 3722, 3687, 3654, 3620, 3588, 3556, 3524,
3493, 3463, 3433, 3404, 3375, 3347, 3319, 3291, 3264, 3238, 3212, 3186, 3161,
3136, 3111, 3087, 3063, 3040, 3017, 2994 };

HuffmanCode::HuffmanCode()
{
	uint i;
	//initialize symbol lookup
	for (i=0;i<256;++i)
	{
		syms[i].code=0;
		syms[i].code_len=0;
		syms[i].value=i;
	}
	std::list <TreeP> nodes;

	for (int w=0;w<129;++w)
	{
//NOTE: this is the static formula the weights are calculated with
//		I've included the results as constant data, because MSVC and GCC
//		produce _different_ results, which breaks data compatibility between
//		compilers. sheesh ...
	
	
/*		float arf;
		if (p<=3) arf=1.0/exp(p);
		else	  arf=1.0/(exp(3)*(1.0+0.125*(p-3)));//+120*p);
		int weight=int(1000000*arf);
*/
	//construct the node list in order of ascending weight
		if (w!=0) nodes.push_front(TreeP(new Tree(256-w,weight[w])));
		if (w!=128) nodes.push_front(TreeP(new Tree(w,weight[w])));
	}

	do
	{
		//grab the two nodes with the lowest weights and combine them
		TreeP a=nodes.front();	nodes.pop_front();
		TreeP b=nodes.front();	nodes.pop_front();

		TreeP merged_nodes=TreeP(new Tree(a.p,b.p));
		//let the tree nodes add traversal bits to the symbol lookup
		a.p->children_add_bit(0,syms);
		b.p->children_add_bit(1,syms);
		
		//insert new parent tree, keeping sort order intact

  		std::list <TreeP>::iterator i=nodes.begin();
  		while ((i!=nodes.end())&&(merged_nodes>(*i))) ++i;
  		nodes.insert(i,merged_nodes);
	} while (nodes.size()>1);

	tree=nodes.front().p;
}

HuffmanCode::~HuffmanCode()
{
	delete(tree);
}
