/*
this is part of brot_core.dll, a fractal generation library
Copyright (C) 2003 Rolf Neuberger

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

/*
changes:
03/Mar-06 * renamed old decompress to decompress_v000 and moved to private
          * added new decompress - Rolf
03/Mar-05 * added save and FILE* constructor - Rolf
03/Mar-05 * added proper LGPL notice, started changelog - Rolf

*/

class
MaskedImage
{
private:
	MaskedImage();				//hide default constructor
public:
	MaskedImage(int width,int height);
	MaskedImage(FILE* inf);
	~MaskedImage();

	bool save(FILE* of);

	//copy constructor
	MaskedImage(const MaskedImage& rhs);
	MaskedImage& operator =(const MaskedImage& rhs);

	const ubyte* get_data();
	//get bit at x/y
	int check_mask(const uint x,const uint y) const { return(1&(mask[stride*y+(x>>3)]>>(x&7))); }
	//set bit at x/y
	void set_mask(const uint x,const uint y) const { mask[stride*y+(x>>3)]|=1<<(x&7); }
	//clear bit at x/y
	void clear_mask(unsigned int x,unsigned int y) const { mask[stride*y+(x>>3)]&=~(1<<(x&7)); }
	//clear all bits
	void clear_mask()const;
	
	void put_and_mask(uint x,uint y,int val) const {image[y*width+x]=ubyte(val); set_mask(x,y); }
	uint get(uint x,uint y) const { return(image[y*width+x]);}

	//fill a rectangle without overwriting masked samples
	void preview_fill(const Section& rect,const uint value) const;
	//draw only the rectangle edges
	void preview_outline(const Section& rect,const uint value) const;

	//fill spans and rectangles without bothering
	void fill(const Span& span,const int value) const;
	void fill(const std::vector <Span>& spans,const int value) const;
	void fill(const Section& rect,const uint value) const;
	uint fill_interior(const Section& rect,const uint value) const;
	
	//move around both the bitmask and image
	void move(int delta_x,int delta_y);

	void decompress();
	void compress();
	int get_used_memory_size()const;
	static int estimate_required_memory(int width,int height);

private:
	void decompress_v000();		//legacy decompression
	void compress_mask();		//bitmask compression, used internally by compress
	void decompress_mask();		//bitmask decompression, used by decompress

private:
	ubyte* mask;				//bitfield storage for mask
	ubyte* image;				//actual image data
	ubyte* compressed_mask;		//same, but compressed to conserve space
	ubyte* compressed_image;
	int width;
	int height;
	int stride;                 //stride in bytes between two lines of the bitmask
	int compressed_size;		//size in bytes of compressed image
	int compressed_mask_size;	//size in bytes of compressed mask
};

