/*
this is part of brot_core.dll, a fractal generation library
Copyright (C) 2003 Rolf Neuberger

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

/*
changes:
03/Mar-05 * added proper LGPL notice, started changelog - Rolf

*/

#include "common.h"
#include "x86_timer.ah"		//needed for Timing.h

#include "Timing.h"

//internal stuff
#include <time.h>

bool AbsTimer::init_done=false;
double AbsTimer::scale=1.0;

AbsTimer::AbsTimer()
{
	if (!init_done)
	{
		x86_timestamp ts_one;
		x86_timestamp ts_two;

		x86_get_timestamp(&ts_one);
		//use the clock function from the standard lib as a reference
		clock_t clock_one=clock();
		clock_t clock_two;

		do
		{
			clock_two=clock();
		} while ((double(clock_two-clock_one)/CLOCKS_PER_SEC)<0.3);
		x86_get_timestamp(&ts_two);

		scale=double(clock_two-clock_one)/CLOCKS_PER_SEC/x86_subtract_timestamps(&ts_one,&ts_two);
		init_done=true;
	}
	reset();
}

void
AbsTimer::reset()
{
	//replace starting timestamp
	x86_get_timestamp(&first_ts);
}

double
AbsTimer::elapsed_seconds()
{
	//get current timestamp, diff against starting timestamp and scale
	x86_timestamp ts;
	x86_get_timestamp(&ts);
	return(scale*x86_subtract_timestamps(&first_ts,&ts));
}

void
RelTimer::reset()
{
	a_time.reset();
}

double
RelTimer::elapsed_seconds()
{
	double ret=a_time.elapsed_seconds();
	a_time.reset();
	return(ret);
}
