/*
this is part of brot2, an example client to the brot_core library
Copyright (C) 2003 Rolf Neuberger

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

/*
changes:
03/Mar-05 * added OFN_HIDEREADONLY flag to 'load context' dialog - Rolf
03/Mar-05 * added 'SHIFT+P' option to reset palette and turn off palette cycling - Rolf
03/Mar-05 * finalized load/save of complete contexts - Rolf
03/Mar-05 * renamed 'crunchy' and 'progress_thingy' to 'blink_status' and 'blink_string' - Rolf
03/Mar-05 * added experimental save - Rolf
03/Mar-05 * added proper LGPL notice, started changelog - Rolf
*/


//#define WIN32_LEAN_AND_MEAN
#include <windows.h>
#include <stdio.h>
#include <gl/glut.h>
#include "common.h"
#include "brot_core_interface.h"
#include "brot_core_cpp_wrapper.h"

char window_title[128];
BrotCoreContext* core=NULL;

char base_path[512];
char places_filename[512];
char pal_filename[512];
ubyte palette[768];
Place place[12];

bool mouse_lock=false;
float gamma=1.75f;
bool background_processing=true;
float palette_cycle_speed=0.0f;



const char blink_string[]=". ";
int blink_status=0;

void
idle_redisplay()
{
	glutPostRedisplay();
}

void
background_crunch()
{
	//stop hogging cycles if calculation is finished
	if ((!background_processing)||
		(!core->refine_history()))
	{
		window_title[0]=' ';
		if (palette_cycle_speed!=0.0f)	glutIdleFunc(idle_redisplay);
		else							glutIdleFunc(NULL);
	}
	else
	{
		++blink_status;
		blink_status&=1;
		window_title[0]=blink_string[blink_status];
	}
	glutSetWindowTitle(window_title);

	if (palette_cycle_speed!=0.0f) glutPostRedisplay();
}

void
crunch()
{
	if (!core->refine_image())
	{
		sprintf(window_title," done    %.1f%% efficiency, %.1f seconds",
			core->get_cleverness()*100,core->get_time_spent());
		//transfer to history refinement if the current view is done
		glutIdleFunc(background_crunch);				
	}
	else
		sprintf(window_title,"%ccrunching - %.1f%%",blink_string[blink_status],100*core->get_progress());

	++blink_status;
	blink_status&=1;

	glutSetWindowTitle(window_title);

	glutPostRedisplay();
}


void
load_context()
{
	char filename[1024];
	filename[0]=0;
	OPENFILENAME crap_stuff;
	memset(&crap_stuff,0,sizeof(crap_stuff));
	crap_stuff.lStructSize=sizeof(crap_stuff);
	crap_stuff.lpstrFilter="Brot core histories\0*.b\0";
	crap_stuff.lpstrFile=filename;
	crap_stuff.nMaxFile=1024;
	crap_stuff.Flags=OFN_HIDEREADONLY;
	crap_stuff.lpstrTitle="Recover your history for a better tomorrow";
	if (GetOpenFileName(&crap_stuff))
	{
		bool me_feel_well=false;		//expect the worst
		//check filename/write privilege
		FILE* file=fopen(filename,"rb");
		if (file)
		{
			//hnkey
			fclose(file);
			me_feel_well=core->load(filename);
		}
		if (!me_feel_well) MessageBox(NULL,"This didn't work too well, I'm afraid :(","*gnarf*",MB_APPLMODAL);
	}
}

void
save_context()
{
	char filename[1024];
	strcpy(filename,"history.b");
	OPENFILENAME crap_stuff;
	memset(&crap_stuff,0,sizeof(crap_stuff));
	crap_stuff.lStructSize=sizeof(crap_stuff);
	crap_stuff.lpstrFilter="Full work progress\0*.b\0Coords and settings only\0*.b\0";
	crap_stuff.lpstrFile=filename;
	crap_stuff.nMaxFile=1024;
	crap_stuff.Flags=OFN_HIDEREADONLY|OFN_OVERWRITEPROMPT;
	crap_stuff.lpstrDefExt="b";
	crap_stuff.lpstrTitle="Save your history today!";
	if (GetSaveFileName(&crap_stuff))
	{
		bool me_feel_well=false;		//expect the worst
		//check filename/write privilege
		FILE* file=fopen(filename,"wb");
		if (file)
		{
			//hnkey
			fclose(file);
			if (crap_stuff.nFilterIndex==1)	me_feel_well=core->save_all(filename);
			else							me_feel_well=core->save_minimum(filename);
		}
		if (!me_feel_well) MessageBox(NULL,"Whoops! Something went wrong!","Oh no ...",MB_APPLMODAL);
	}
}


void
reshape(int x,int y)
{
	glViewport(0,0,x,y);
	core->set_image_properties(x,y,-1,-1.0f);
	glutIdleFunc(crunch);
}

void
display()
{
	glRasterPos2f(-1.0f,1.0f);
	glPixelZoom(1,-1);
	glPixelStorei(GL_UNPACK_ALIGNMENT,1);
	
//	const ubyte* gnarf=core->get_rgb_image();

	glDrawPixels(core->get_width(),
				 core->get_height(),
				 GL_RGB,GL_UNSIGNED_BYTE,
				 core->get_rgb_image());

	glutSwapBuffers();
}

void
show_params()
{
	double real,img,radius;
	core->get_view_params(&real,&img,&radius);
	printf("mandelbrot center = %.10f / %.10f\nradius= %.5g; ",real,img,radius);
	printf("maximum iteration depth = %u\n",core->get_maximum_iterations());
}

void
load_palette()
{
	char filename[1024];
	filename[0]=0;
	OPENFILENAME crap_stuff;
	memset(&crap_stuff,0,sizeof(crap_stuff));
	crap_stuff.lStructSize=sizeof(crap_stuff);
	crap_stuff.lpstrFilter="Palettes\0*.PAL\0All files\0*.*\0";
	crap_stuff.lpstrFile=filename;
	crap_stuff.nMaxFile=1024;
	crap_stuff.lpstrInitialDir=base_path;
	crap_stuff.lpstrTitle="Select a new palette, baby";
	if (GetOpenFileName(&crap_stuff))
	{
		FILE* pal_file=fopen(filename,"rb");
		if (pal_file)
		{
			fread(palette,256,3,pal_file);
			fclose(pal_file);
			core->set_palette(palette);
		}
	}
}

void
save_as_TGA()
{
	struct TGA_header
	{
		short dum,code,dee,dah,duh,doing,x,y,bpp;
	} header;
	memset(&header,0,sizeof(header));
	header.code=3;
	header.x=core->get_width();
	header.y=core->get_height();
	header.bpp=24;
	const ubyte* img=core->get_rgb_image();

	if (NULL==img) return;

	char filename[1024];
	strcpy(filename,"yipiieh.TGA");
	OPENFILENAME crap_stuff;
	memset(&crap_stuff,0,sizeof(crap_stuff));
	crap_stuff.lStructSize=sizeof(crap_stuff);
	crap_stuff.lpstrFilter="TGA images\0*.TGA\0";
	crap_stuff.lpstrFile=filename;
	crap_stuff.nMaxFile=1024;
	crap_stuff.Flags=OFN_HIDEREADONLY|OFN_OVERWRITEPROMPT;
	crap_stuff.lpstrDefExt="TGA";
	crap_stuff.lpstrTitle="Give me a name ... save me!";
	if (GetSaveFileName(&crap_stuff))
	{
		FILE* file=fopen(filename,"wb");
		if (file)
		{
			fwrite(&header,sizeof(header),1,file);
			//grrr, byte swap
			ubyte* line=(ubyte*)malloc(3*header.x);
			for (int y=0;y<header.y;++y)
			{
				for (int x=0;x<3*header.x;x+=3)
				{
					line[x]=img[x+2];
					line[x+1]=img[x+1];
					line[x+2]=img[x];
				}
				img+=3*header.x;
				fwrite(line,3*header.x,1,file);
			}
			free(line);
		}
		fclose(file);
	}
}

void
mouse(int button,int state,int x,int y)
{
	if (mouse_lock)
	{
		strcpy(window_title+1,"Press L first.");
		glutSetWindowTitle(window_title);
		return;
	}

	if (GLUT_UP==state)
	{
		if (GLUT_LEFT_BUTTON==button)
		{
			if (glutGetModifiers()&GLUT_ACTIVE_SHIFT)
					core->zoom(x,y,16.0f);
			else	core->zoom(x,y,2.0f);
		}
		else
		if (GLUT_RIGHT_BUTTON==button)
		{
			if (glutGetModifiers()&GLUT_ACTIVE_SHIFT)
					core->zoom(x,y,0.0625f);
			else	core->zoom(x,y,0.5f);
		}
		show_params();
		glutIdleFunc(crunch);
	}
}

void
special_keys(int key,int x,int y)
{
	if ((key>=GLUT_KEY_F1)&&(key<=GLUT_KEY_F12))
	{
		uint place_num=key-GLUT_KEY_F1;
		if (glutGetModifiers()&GLUT_ACTIVE_CTRL)
		{
			core->get_view_params(&place[place_num]);
			printf("saved this location to slot #%u\n",place_num+1);
		}
		else
		{
			core->set_view_params(&place[place_num]);
			printf("restored location #%u\n",place_num+1);
			show_params();
			glutIdleFunc(crunch);
		}
	}
	if (key==GLUT_KEY_PAGE_UP)
	{
		if (glutGetModifiers()&GLUT_ACTIVE_SHIFT)	core->zoom(2.56f);
		else										core->zoom(1.6f);
		show_params();
		glutIdleFunc(crunch);
	}
	if (key==GLUT_KEY_PAGE_DOWN)
	{
		if (glutGetModifiers()&GLUT_ACTIVE_SHIFT)	core->zoom(0.390625f);
		else										core->zoom(0.625f);
		show_params();
		glutIdleFunc(crunch);
	}
	float move_speed;
	if (glutGetModifiers()&GLUT_ACTIVE_SHIFT)	move_speed=0.3125f;
	else										move_speed=0.125f;
	if (key==GLUT_KEY_LEFT)
	{
		core->move(-move_speed,0.0f);
		show_params();
		glutIdleFunc(crunch);
	}
	if (key==GLUT_KEY_RIGHT)
	{
		core->move(move_speed,0.0f);
		show_params();
		glutIdleFunc(crunch);
	}
	if (key==GLUT_KEY_UP)
	{
		core->move(0.0f,move_speed);
		show_params();
		glutIdleFunc(crunch);
	}
	if (key==GLUT_KEY_DOWN)
	{
		core->move(0.0f,-move_speed);
		show_params();
		glutIdleFunc(crunch);
	}
}

void
keyboard(unsigned char key,int x,int y)
{
	if (key=='0')		//increase aa gamma correction reference value
	{
		gamma+=0.125f;
		if (gamma>2.5f) gamma=2.5f;
		else
		{
			core->set_image_properties(-1,-1,-1,gamma);
			printf("aa gamma correction set to %.3f\n",gamma);
			glutPostRedisplay();
		}
	}
	if (key=='9')		//decrease aa gamma correction reference value
	{
		gamma-=0.125f;
		if (gamma<0.5f) gamma=0.5f;
		else
		{
			core->set_image_properties(-1,-1,-1,gamma);
			printf("aa gamma correction set to %.3f\n",gamma);
			glutPostRedisplay();
		}
	}

	if ((key>='1')&&(key<='8'))		//set supersampling strength
	{
		int aa_strength=(key-'1')+1;
		core->set_image_properties(-1,-1,aa_strength,-1.0f);
		if (aa_strength==1)	printf("no antialiasing\n");
		else				printf("%ux%u ordered grid supersampling\n",aa_strength,aa_strength);
		glutIdleFunc(crunch);
	}
	if ('Q'==(key&(~0x20)))
	{
		if (key&0x20)	core->adjust_maximum_iterations(2.0f);
		else			core->adjust_maximum_iterations(1.25f);
		printf("max iterations=%u\n",core->get_maximum_iterations());
		glutIdleFunc(crunch);
	}
	if ('A'==(key&(~0x20)))
	{
		if (key&0x20)	core->adjust_maximum_iterations(0.5f);
		else			core->adjust_maximum_iterations(0.8f);
		printf("max iterations=%u\n",core->get_maximum_iterations());
		glutIdleFunc(crunch);
	}
	//change palette cycling speed
	if (key=='c')		//increase
	{
		if (palette_cycle_speed<256.0f)
		{
			if (palette_cycle_speed==0.0f) palette_cycle_speed=8.0f;
			else
			if (palette_cycle_speed>=8.0f) palette_cycle_speed*=1.5f;
			else
			{
				if (palette_cycle_speed>=-8.0f) palette_cycle_speed=0.0f;
				else palette_cycle_speed*=0.6666666f;
			}
			core->set_palette_cycling_speed(palette_cycle_speed);
			glutIdleFunc(crunch);
		}
	}

	if (key=='C')		//decrease
	{
		if (palette_cycle_speed>-256.0f)
		{
			if (palette_cycle_speed==0.0f) palette_cycle_speed=-8.0f;
			else
			if (palette_cycle_speed<=-8.0f) palette_cycle_speed*=1.5f;
			else
			{
				if (palette_cycle_speed<=8.0f) palette_cycle_speed=0.0f;
				else palette_cycle_speed*=0.6666666f;
			}
			core->set_palette_cycling_speed(palette_cycle_speed);
			glutIdleFunc(crunch);
		}
	}

	if ('p'==key)
	{
		load_palette();
		glutIdleFunc(crunch);
	}
	
	if ('P'==key)
	{
		printf("Resetting palette\n");
		palette_cycle_speed=0.0f;
		core->set_palette_cycling_speed(0.0f);
		core->set_palette(palette);
		glutIdleFunc(crunch);
	}

	if (key=='H')
	{
		save_context();
	}

	if (key=='h')
	{
		load_context();
		//loaded current view may have different resolution
		glutReshapeWindow(core->get_width(),core->get_height());
		show_params();
		glutIdleFunc(crunch);
	}

	//convert to upper case for remaining options
	key&=~0x20;

	if (key=='Y')
	{
		if (core->history_back())
		{
			//the previous history entry may have a different image size
			glutReshapeWindow(core->get_width(),core->get_height());
			show_params();
			glutIdleFunc(crunch);
		}
	}
	if (key=='X')
	{
		if (core->history_forward())
		{
			//the next history entry may have a different image size
			glutReshapeWindow(core->get_width(),core->get_height());
			show_params();
			glutIdleFunc(crunch);
		}
	}

	if (0==key)	//space bar
	{
		core->set_view_params(0.0,0.0,2.0);
		show_params();
		glutIdleFunc(crunch);
	}
	if ('L'==key)
	{
		mouse_lock=!mouse_lock;
		if (mouse_lock) strcpy(window_title+1,"Mouse lock active");
		else			strcpy(window_title+1,"Mouse lock off");
		glutSetWindowTitle(window_title);
	}
	if ('M'==key)
	{
		int type;
		do
		{
			type=core->change_generator_type();
		} while(type==BC_BRUTE_FORCE);
		printf("using %s\n",core->describe_generator(type));
		glutIdleFunc(crunch);
	}
	if ('D'==key)
	{
		if (core->toggle_preview())	printf("preview enabled\n");
		else						printf("preview disabled\n");
	}
	if ('B'==key)
	{
		background_processing=!background_processing;
		if (background_processing)		printf("background crunching on\n");
		else							printf("background crunching off\n");
		glutIdleFunc(crunch);
	}
	if ('R'==key)
	{
		core->reset_current_view();
		printf("restarting\n");
		glutIdleFunc(crunch);
	}
	if ('S'==key)
	{
		save_as_TGA();
	}
}

void
cleanup()
{
	FILE* places_f=fopen(places_filename,"wb");
	if (places_f)
	{
		for (uint i=0;i<12;++i)
		{
			fwrite(&place[i],sizeof(Place),1,places_f);
		}
		fclose(places_f);
	}
	SAFE_DELETE(core);
}

int
main(int argc,char** argv)
{
	strcpy(window_title,"hnngrgrl");
	printf("Deep zoom by zeckensack *hrhr*\n\n");
	printf("This was getting overly long, so have a look at the readme for controls.\n\n");
	//load favourites

//dump command line ('support issues')
	printf("Command line:\n");
	for (int dump=0;dump<argc;++dump)
	{
		printf(argv[dump]);
		if (dump==(argc-1)) printf("\n");
		else				printf(" ");
	}
	//find base path
	strcpy(base_path,argv[0]);	//copy exe name
	int path_len=strlen(base_path);
	bool is_path=false;
	for (int i=path_len-1;i>0;--i)
	{
		if (base_path[i]==':') is_path=true;
		if ((base_path[i-1]=='\\')||(base_path[i-1]=='/'))
		{
			is_path=true;
			base_path[i]=0;
			break;
		}
	}
	//workaround for 2k, path to exe may not be qualified
	if (!is_path) base_path[0]=0;


	strcpy(places_filename,base_path);
	strcat(places_filename,"places");
	strcpy(pal_filename,base_path);
	strcat(pal_filename,"def.pal");

	int megs=96;
	char* mem_argument=NULL;
	for (int p=1;p<argc;++p)
	{
		mem_argument=strstr(argv[p],"-megs=");
		if (mem_argument)
		{
			megs=atoi(mem_argument+6);
		}
		else
		{
			if (NULL==strchr(argv[p],'\\'))
			{
				strcpy(pal_filename,base_path);
				strcat(pal_filename,argv[p]);
			}
			else
				strcpy(pal_filename,argv[p]);
		}
	}
	if (megs<0) megs=96;
		
	printf("memory limit is %u megabytes\n",megs);
	core=new BrotCoreContext(megs);

	core->set_image_properties(600,600,1,gamma);
	atexit(cleanup);

	FILE* places_f=fopen(places_filename,"rb");
	if (places_f)
	{
		printf("Loading places: ");
		printf(places_filename);
		printf("\n");
		for (uint i=0;i<12;++i)
		{
			fread(&place[i],sizeof(Place),1,places_f);
		}
		fclose(places_f);
	}
	else
	{
		printf("No preset places found\n");
		for (uint i=0;i<12;++i)
		{
			core->get_view_params(&place[i]);
		}
	}
	printf("loading palette: %s\n",pal_filename);
	FILE* pal_file=fopen(pal_filename,"rb");
	if (pal_file)
	{
		fread(palette,256,3,pal_file);
		fclose(pal_file);
	}
	core->set_palette(palette);

	glutInit(&argc,argv);
	glutInitWindowSize(600,600);
	glutInitDisplayMode(GLUT_RGB|GLUT_DOUBLE);

	glutCreateWindow(window_title);

	glutDisplayFunc(display);
	glutKeyboardFunc(keyboard);
	glutSpecialFunc(special_keys);
	glutReshapeFunc(reshape);
	glutMouseFunc(mouse);

	glutMainLoop();				//_never_ returns

	//we won't get here, this merely kills a compiler warning
	return(0);
}

