/*
this is part of brot_core.dll, a fractal generation library
Copyright (C) 2003 Rolf Neuberger

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

/*
changes:
03/Mar-05 * added proper LGPL notice, started changelog - Rolf

*/

class
RgbImage
{
private:
	RgbImage();	//hide default constructor
public:
	RgbImage(int width,int height);
	~RgbImage();

	void set_size(int width,int height);
	void set_aa_params(float gamma,int aa_degree);

	void convert_with_palette(const ubyte* const indexed_image);
	void set_palette(const ubyte* palette);
	
	void cycle_palette(int cycles);

	int get_width()	{ return(width); }
	int get_height() { return(height); }
	ubyte* get_rgb_data() { return(data); }

	void force_decode() { must_decode=true;}
	void notify_image_change() { should_decode=true; }

	void save_palette(FILE* of);

private:
	void prep_subsample_info();
	void massage_palette();	//applies the degamma function to the palette and writes it to ...
private:
	downfilter_info di;		//data struct for the assembly downfiltering (see downfilter.ah)

	ubyte palette[768];
	ushort degamma_lut[256];

	AbsTimer timer;
	float slack;

	int width;
	int height;
	ubyte* data;
	float gamma;
	int aa_degree;
	bool must_decode;
	bool should_decode;
};

