;this is part of brot_core.dll, a fractal generation library
;Copyright (C) 2003 Rolf Neuberger
;
;This library is free software; you can redistribute it and/or
;modify it under the terms of the GNU Lesser General Public
;License as published by the Free Software Foundation; either
;version 2.1 of the License, or (at your option) any later version.
;
;This library is distributed in the hope that it will be useful,
;but WITHOUT ANY WARRANTY; without even the implied warranty of
;MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
;Lesser General Public License for more details.
;
;You should have received a copy of the GNU Lesser General Public
;License along with this library; if not, write to the Free Software
;Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA


;convert an IEEE-754 double to an extremefloat
;prototype:
;void x86_double_to_xfloat(ExtremFloat* dest,double* src)

_x86_double_to_xfloat:
	PUSH EAX
	PUSH EBX
	PUSH ECX
	PUSH EDX
	PUSH ESI
	MOV ESI,[byte ESP+24]
	MOV EDX,[byte ESP+28]
	MOV EAX,[EDX]			;load source
	MOV EBX,[byte EDX+4]
	MOV EDX,EBX
	TEST EBX,EBX			;test sign bit
	SETS CL
	MOVZX ECX,CL			;expand to wasteful DWORD
	MOV [byte ESI+28],ECX	;store sign
	XOR ECX,ECX
	MOV [ESI],ECX			;wipe low 128 bits of mantissa
	MOV [byte ESI+4],ECX
	MOV [byte ESI+8],ECX
	MOV [byte ESI+12],ECX
	ADD EDX,EDX				;discard sign and check exponent/high bits of mantissa
	JZ .zero_or_denormal
	SHR EDX,byte 21			;move exponent to lsbs
	SUB EDX,1023			;remove exponent bias
	MOV [byte ESI+24],EDX	;store exponent
	SHLD EBX,EAX,11			;adjust mantissa
	SHL EAX,11
	OR EBX,0x80000000		;restore 'hidden bit'
	MOV [byte ESI+16],EAX
	MOV [byte ESI+20],EBX	;store mantissa
.done:
	POP ESI
	POP EDX
	POP ECX
	POP EBX
	POP EAX
	RETN
.zero_or_denormal:
	MOV [byte ESI+16],ECX
	MOV [byte ESI+20],ECX
	MOV [byte ESI+24],ECX
	MOV [byte ESI+28],ECX
	JMP .done

;convert a 32 bit signed integer to an extremefloat
;prototype:
;void __cdecl x86_int_to_xfloat(ExtremeFloat* dest,int src)
_x86_int_to_xfloat:
	PUSH EAX
	PUSH EBX
	PUSH ECX
	PUSH EDX
	PUSH EDI
	MOV EDI,[byte ESP+24]
	XOR EDX,EDX
	MOV [byte EDI+28],EDX	;clear sign bit
	MOV EAX,[byte ESP+28]
	TEST EAX,EAX
	JZ _x86_uint_to_xfloat.value_is_zero
	JNS _x86_uint_to_xfloat.non_negative

;value is negative
	INC dword [byte EDI+28]	;set sign bit
	NEG EAX					;invert to get absolute value
	JMP _x86_uint_to_xfloat.non_negative

;convert a 32 bit _un_signed integer to an extremefloat
;prototype:
;void __cdecl x86_uint_to_xfloat(ExtremeFloat* dest,uint src)
_x86_uint_to_xfloat:
	PUSH EAX
	PUSH EBX
	PUSH ECX
	PUSH EDX
	PUSH EDI
	MOV EDI,[byte ESP+24]
	XOR EDX,EDX
	MOV [byte EDI+28],EDX	;clear sign bit
	MOV EAX,[byte ESP+28]
	TEST EAX,EAX
	JZ .value_is_zero
.non_negative:
	BSR EBX,EAX				;find MSB
	MOV ECX,31
	SUB ECX,EBX
	SHL EAX,CL				;normalize mantissa
	MOV [byte EDI+20],EAX
	MOV [byte EDI+24],EBX	;write exponent
	JMP .finished
.value_is_zero:
	MOV [byte EDI+20],EDX	;EDX is still zero
	MOV [byte EDI+24],EDX
.finished:
	MOV [EDI],EDX			;wipe low mantissa bits
	MOV [byte EDI+4],EDX
	MOV [byte EDI+8],EDX
	MOV [byte EDI+12],EDX
	MOV [byte EDI+16],EDX
	POP EDI
	POP EDX
	POP ECX
	POP EBX
	POP EAX
	RETN


;convert an extremefloat back to double
;prototype:
;double __cdecl x86_xfloat_to_double(const ExtremeFloat* src)
_x86_xfloat_to_double:
	PUSH EAX
	PUSH ECX
	PUSH EDX
	PUSH ESI
	MOV ESI,[byte ESP+20]
	XOR EDX,EDX
	MOV EAX,[byte ESI+20]
	TEST EAX,EAX
	JZ .value_is_zero
	AND EAX,0x7FFFFFFF
	MOV EDX,[byte ESI+16]
	SHRD EDX,EAX,11
	SHR EAX,11
	MOV ECX,dword [byte ESI+24]	;load exponent
	ADD ECX,1023				;add exponent bias
	SHL ECX,20					;shift to exponent position for double
	OR EAX,ECX
	MOV ECX,[byte ESI+28]		;load sign
	SHL ECX,31
	OR EAX,ECX					;insert sign into result
.value_is_zero:
	SUB ESP,byte 8				;allocate temp storage on stack
	MOV [ESP],EDX
	MOV [byte ESP+4],EAX
	FLD qword [ESP]
	ADD ESP,byte 8
	POP ESI
	POP EDX
	POP ECX
	POP EAX
	RETN

;return order of absolute magnitudes - 0 means equal magnitude
;1 means that abs(left)>abs(right) ### 2 means abs(right)>abs(left)
;prototype:
;int x86_xfloat_abs_order(const ExtremeFloat* left,const ExtremeFloat* right)
_x86_xfloat_abs_order:
	PUSH EBX
	PUSH ECX
	PUSH EDX
	PUSH ESI
	PUSH EDI
	MOV ESI,[byte ESP+24]
	MOV EDI,[byte ESP+28]
	XOR EAX,EAX				;default return value is zero, meaning abs(left)==abs(right)
	MOV EBX,[byte ESI+24]	;compare exponents
	CMP EBX,[byte EDI+24]
	JG .left_greater
	JL .right_greater

;exponents are equal
	MOV ECX,3
.compare_mantissa_loop:
	DEC ECX
	JC .equal
	MOV EBX,[byte ESI+8*ECX+4]
	CMP EBX,[byte EDI+8*ECX+4]
	JA .left_greater
	JB .right_greater
	MOV EDX,[byte ESI+8*ECX]
	CMP EDX,[byte EDI+8*ECX]
	JZ .compare_mantissa_loop
	JA .left_greater

.right_greater:
	INC EAX				;return 2, meaning abs(right)>abs(left)
.left_greater:
	INC EAX				;return 1, meaning abs(left)>abs(right)
		
.equal:
	POP EDI
	POP ESI
	POP EDX
	POP ECX
	POP EBX
	RETN

